@@ -21,6 +21,7 @@ use super::Result;
 
 #[allow(unused_imports)]
 use cl3::ext;
+#[allow(unused_imports)]
 use cl3::types::{
     cl_context, cl_device_id, cl_event, cl_kernel, cl_kernel_exec_info, cl_program, cl_uint,
     cl_ulong,
@@ -120,6 +121,7 @@ impl Kernel {
     /// * `arg_ptr` - the SVM pointer to the data for the argument at arg_index.
     ///
     /// returns an empty Result or the error code from the OpenCL C API function.
+    #[cfg(feature = "CL_VERSION_2_0")]
     pub fn set_arg_svm_pointer(&self, arg_index: cl_uint, arg_ptr: *const c_void) -> Result<()> {
         Ok(set_kernel_arg_svm_pointer(self.kernel, arg_index, arg_ptr)?)
     }
@@ -131,6 +133,7 @@ impl Kernel {
     /// * `param_ptr` - pointer to the data for the param_name.
     ///
     /// returns an empty Result or the error code from the OpenCL C API function.
+    #[cfg(feature = "CL_VERSION_2_0")]
     pub fn set_exec_info<T>(
         &self,
         param_name: cl_kernel_exec_info,
@@ -168,6 +171,7 @@ impl Kernel {
         Ok(get_kernel_info(self.kernel, KernelInfo::CL_KERNEL_ATTRIBUTES)?.to_string())
     }
 
+    #[cfg(feature = "CL_VERSION_1_2")]
     pub fn get_arg_address_qualifier(&self, arg_indx: cl_uint) -> Result<cl_uint> {
         Ok(get_kernel_arg_info(
             self.kernel,
@@ -177,6 +181,7 @@ impl Kernel {
         .to_uint())
     }
 
+    #[cfg(feature = "CL_VERSION_1_2")]
     pub fn get_arg_access_qualifier(&self, arg_indx: cl_uint) -> Result<cl_uint> {
         Ok(get_kernel_arg_info(
             self.kernel,
@@ -186,6 +191,7 @@ impl Kernel {
         .to_uint())
     }
 
+    #[cfg(feature = "CL_VERSION_1_2")]
     pub fn get_arg_type_qualifier(&self, arg_indx: cl_uint) -> Result<cl_ulong> {
         Ok(get_kernel_arg_info(
             self.kernel,
@@ -195,6 +201,7 @@ impl Kernel {
         .to_ulong())
     }
 
+    #[cfg(feature = "CL_VERSION_1_2")]
     pub fn get_arg_type_name(&self, arg_indx: cl_uint) -> Result<String> {
         Ok(get_kernel_arg_info(
             self.kernel,
@@ -204,6 +211,7 @@ impl Kernel {
         .to_string())
     }
 
+    #[cfg(feature = "CL_VERSION_1_2")]
     pub fn get_arg_name(&self, arg_indx: cl_uint) -> Result<String> {
         Ok(
             get_kernel_arg_info(self.kernel, arg_indx, KernelArgInfo::CL_KERNEL_ARG_NAME)?
@@ -367,7 +375,7 @@ impl<'a> ExecuteKernel<'a> {
     /// * `size` - the size of the local memory buffer in bytes.
     ///
     /// returns a reference to self.
-    pub fn set_arg_local_buffer<'b, T>(&'b mut self, size: size_t) -> &'b mut Self {
+    pub fn set_arg_local_buffer<T>(&mut self, size: size_t) -> &mut Self {
         assert!(
             self.arg_index < self.num_args,
             "ExecuteKernel::set_arg_local_buffer too many args"
@@ -390,7 +398,8 @@ impl<'a> ExecuteKernel<'a> {
     /// * `arg` - a reference to the data for the kernel argument.
     ///
     /// returns a reference to self.
-    pub fn set_arg_svm<'b, T>(&'b mut self, arg_ptr: *const T) -> &'b mut Self {
+    #[cfg(feature = "CL_VERSION_2_0")]
+    pub fn set_arg_svm<T>(&mut self, arg_ptr: *const T) -> &mut Self {
         assert!(
             self.arg_index < self.num_args,
             "ExecuteKernel::set_arg_svm too many args"
@@ -410,11 +419,12 @@ impl<'a> ExecuteKernel<'a> {
     /// * `param_ptr` - pointer to the data for the param_name.
     ///
     /// returns a reference to self.
-    pub fn set_exec_info<'b, T>(
-        &'b mut self,
+    #[cfg(feature = "CL_VERSION_2_0")]
+    pub fn set_exec_info<T>(
+        &mut self,
         param_name: cl_kernel_exec_info,
         param_ptr: *const T,
-    ) -> &'b mut Self {
+    ) -> &mut Self {
         self.kernel.set_exec_info(param_name, param_ptr).unwrap();
         self
     }
@@ -424,7 +434,7 @@ impl<'a> ExecuteKernel<'a> {
     /// * `size` - the size of the global work offset.
     ///
     /// returns a reference to self.
-    pub fn set_global_work_offset<'b>(&'b mut self, size: size_t) -> &'b mut Self {
+    pub fn set_global_work_offset(&mut self, size: size_t) -> &mut Self {
         self.global_work_offsets.push(size);
         self
     }
@@ -438,7 +448,7 @@ impl<'a> ExecuteKernel<'a> {
     /// * `sizes` - the sizes of the global work offset.
     ///
     /// returns a reference to self.
-    pub fn set_global_work_offsets<'b>(&'b mut self, sizes: &[size_t]) -> &'b mut Self {
+    pub fn set_global_work_offsets(&mut self, sizes: &[size_t]) -> &mut Self {
         assert!(
             self.global_work_offsets.is_empty(),
             "ExecuteKernel::set_global_work_offsets already set"
@@ -453,7 +463,7 @@ impl<'a> ExecuteKernel<'a> {
     /// * `size` - the size of the global work size.
     ///
     /// returns a reference to self.
-    pub fn set_global_work_size<'b>(&'b mut self, size: size_t) -> &'b mut Self {
+    pub fn set_global_work_size(&mut self, size: size_t) -> &mut Self {
         self.global_work_sizes.push(size);
         self
     }
@@ -482,7 +492,7 @@ impl<'a> ExecuteKernel<'a> {
     /// * `size` - the size of the local work size.
     ///
     /// returns a reference to self.
-    pub fn set_local_work_size<'b>(&'b mut self, size: size_t) -> &'b mut Self {
+    pub fn set_local_work_size(&mut self, size: size_t) -> &mut Self {
         self.local_work_sizes.push(size);
         self
     }
@@ -692,36 +702,39 @@ mod tests {
         println!("kernel.attributes(): {}", value);
         // assert!(value.is_empty());
 
-        let arg0_address = kernels[0]
-            .get_arg_address_qualifier(0)
-            .expect("OpenCL kernel_0.get_arg_address_qualifier");
-        println!(
-            "OpenCL kernel_0.get_arg_address_qualifier: {:X}",
-            arg0_address
-        );
+        #[cfg(feature = "CL_VERSION_1_2")]
+        {
+            let arg0_address = kernels[0]
+                .get_arg_address_qualifier(0)
+                .expect("OpenCL kernel_0.get_arg_address_qualifier");
+            println!(
+                "OpenCL kernel_0.get_arg_address_qualifier: {:X}",
+                arg0_address
+            );
 
-        let arg0_access = kernels[0]
-            .get_arg_access_qualifier(0)
-            .expect("OpenCL kernel_0.get_arg_access_qualifier");
-        println!(
-            "OpenCL kernel_0.get_arg_access_qualifier: {:X}",
-            arg0_access
-        );
+            let arg0_access = kernels[0]
+                .get_arg_access_qualifier(0)
+                .expect("OpenCL kernel_0.get_arg_access_qualifier");
+            println!(
+                "OpenCL kernel_0.get_arg_access_qualifier: {:X}",
+                arg0_access
+            );
 
-        let arg0_type_name = kernels[0]
-            .get_arg_type_name(0)
-            .expect("OpenCL kernel_0.get_arg_type_name");
-        println!("OpenCL kernel_0.get_arg_type_name: {}", arg0_type_name);
+            let arg0_type_name = kernels[0]
+                .get_arg_type_name(0)
+                .expect("OpenCL kernel_0.get_arg_type_name");
+            println!("OpenCL kernel_0.get_arg_type_name: {}", arg0_type_name);
 
-        let arg0_type = kernels[0]
-            .get_arg_type_qualifier(0)
-            .expect("OpenCL kernel_0.get_arg_type_qualifier");
-        println!("OpenCL kernel_0.get_arg_type_qualifier: {}", arg0_type);
+            let arg0_type = kernels[0]
+                .get_arg_type_qualifier(0)
+                .expect("OpenCL kernel_0.get_arg_type_qualifier");
+            println!("OpenCL kernel_0.get_arg_type_qualifier: {}", arg0_type);
 
-        let arg0_name = kernels[0]
-            .get_arg_name(0)
-            .expect("OpenCL kernel_0.get_arg_name");
-        println!("OpenCL kernel_0.get_arg_name: {}", arg0_name);
+            let arg0_name = kernels[0]
+                .get_arg_name(0)
+                .expect("OpenCL kernel_0.get_arg_name");
+            println!("OpenCL kernel_0.get_arg_name: {}", arg0_name);
+        }
 
         let value = kernels[0].get_work_group_size(device.id()).unwrap();
         println!("kernel.get_work_group_size(): {}", value);
