@@ -25,6 +25,7 @@ use cl3::ext;
 use cl3::types::{cl_context, cl_device_id, cl_int, cl_program, cl_uchar, cl_uint, CL_FALSE};
 #[allow(unused_imports)]
 use libc::{c_void, intptr_t, size_t};
+#[allow(unused_imports)]
 use std::ffi::{CStr, CString};
 use std::ptr;
 use std::result;
@@ -168,6 +169,7 @@ impl Program {
     ///
     /// returns a Result containing the new Program
     /// or the error code from the OpenCL C API function.
+    #[cfg(feature = "CL_VERSION_1_2")]
     pub fn create_from_builtin_kernels(
         context: &Context,
         devices: &[cl_device_id],
@@ -219,7 +221,7 @@ impl Program {
     pub fn build(&mut self, devices: &[cl_device_id], options: &str) -> Result<()> {
         // Ensure options string is null terminated
         let c_options = CString::new(options).expect("Program::build, invalid options");
-        build_program(self.program, &devices, &c_options, None, ptr::null_mut())?;
+        build_program(self.program, devices, &c_options, None, ptr::null_mut())?;
         self.kernel_names = self.get_kernel_names()?;
         Ok(())
     }
@@ -239,7 +241,7 @@ impl Program {
         options: &str,
     ) -> result::Result<Program, String> {
         let mut program =
-            Program::create_from_sources(&context, sources).map_err(|e| e.to_string())?;
+            Program::create_from_sources(context, sources).map_err(|e| e.to_string())?;
         match program.build(context.devices(), options) {
             Ok(_) => Ok(program),
             Err(e) => {
@@ -269,9 +271,7 @@ impl Program {
         options: &str,
     ) -> result::Result<Program, String> {
         let sources = [src];
-        Ok(Program::create_and_build_from_sources(
-            context, &sources, options,
-        )?)
+        Program::create_and_build_from_sources(context, &sources, options)
     }
 
     /// Create and build an OpenCL Program from binaries with the given options.  
@@ -287,7 +287,7 @@ impl Program {
         binaries: &[&[u8]],
         options: &str,
     ) -> Result<Program> {
-        let mut program = Program::create_from_binary(&context, context.devices(), binaries)?;
+        let mut program = Program::create_from_binary(context, context.devices(), binaries)?;
         program.build(context.devices(), options)?;
         Ok(program)
     }
@@ -323,6 +323,7 @@ impl Program {
     ///
     /// returns a null Result
     /// or the error code from the OpenCL C API function.
+    #[cfg(feature = "CL_VERSION_1_2")]
     pub fn compile(
         &mut self,
         devices: &[cl_device_id],
@@ -334,10 +335,10 @@ impl Program {
         let c_options = CString::new(options).expect("Program::compile, invalid options");
         Ok(compile_program(
             self.program,
-            &devices,
+            devices,
             &c_options,
-            &input_headers,
-            &header_include_names,
+            input_headers,
+            header_include_names,
             None,
             ptr::null_mut(),
         )?)
@@ -352,6 +353,7 @@ impl Program {
     ///
     /// returns a null Result
     /// or the error code from the OpenCL C API function.
+    #[cfg(feature = "CL_VERSION_1_2")]
     pub fn link(
         &mut self,
         devices: &[cl_device_id],
@@ -362,9 +364,9 @@ impl Program {
         let c_options = CString::new(options).expect("Program::link, invalid options");
         self.program = link_program(
             self.program,
-            &devices,
+            devices,
             &c_options,
-            &input_programs,
+            input_programs,
             None,
             ptr::null_mut(),
         )?;
@@ -453,12 +455,12 @@ impl Program {
         Ok(get_program_info(self.program, ProgramInfo::CL_PROGRAM_KERNEL_NAMES)?.to_string())
     }
 
-    #[cfg(feature = "CL_VERSION_2_1")]
+    /// CL_VERSION_2_1
     pub fn get_program_il(&self) -> Result<String> {
         Ok(get_program_info(self.program, ProgramInfo::CL_PROGRAM_IL)?.to_string())
     }
 
-    #[cfg(feature = "CL_VERSION_2_2")]
+    /// CL_VERSION_2_2
     pub fn get_program_scope_global_ctors_present(&self) -> Result<bool> {
         Ok(get_program_info(
             self.program,
@@ -468,7 +470,7 @@ impl Program {
             != CL_FALSE)
     }
 
-    #[cfg(feature = "CL_VERSION_2_2")]
+    /// CL_VERSION_2_2
     pub fn get_program_scope_global_dtors_present(&self) -> Result<bool> {
         Ok(get_program_info(
             self.program,
@@ -512,7 +514,7 @@ impl Program {
         .to_uint())
     }
 
-    #[cfg(feature = "CL_VERSION_2_0")]
+    /// CL_VERSION_2_0
     pub fn get_build_global_variable_total_size(&self, device: cl_device_id) -> Result<size_t> {
         Ok(get_program_build_info(
             self.program,
