@@ -13,16 +13,20 @@
 // limitations under the License.
 
 use super::context::Context;
+use super::Result;
 
 use cl3::device::{
-    CL_DEVICE_SVM_COARSE_GRAIN_BUFFER, CL_DEVICE_SVM_FINE_GRAIN_BUFFER,
+    CL_DEVICE_SVM_ATOMICS, CL_DEVICE_SVM_COARSE_GRAIN_BUFFER, CL_DEVICE_SVM_FINE_GRAIN_BUFFER,
     CL_DEVICE_SVM_FINE_GRAIN_SYSTEM,
 };
-use cl3::memory::{svm_alloc, svm_free, CL_MEM_READ_WRITE, CL_MEM_SVM_FINE_GRAIN_BUFFER};
+use cl3::memory::{
+    svm_alloc, svm_free, CL_MEM_READ_WRITE, CL_MEM_SVM_ATOMICS, CL_MEM_SVM_FINE_GRAIN_BUFFER,
+};
 use cl3::types::{cl_device_svm_capabilities, cl_svm_mem_flags, cl_uint};
 use libc::c_void;
 use std::fmt;
 use std::fmt::Debug;
+use std::iter::IntoIterator;
 use std::marker::PhantomData;
 use std::mem;
 use std::ops::{Deref, DerefMut};
@@ -33,6 +37,7 @@ struct SvmRawVec<'a, T> {
     cap: usize,
     context: &'a Context,
     fine_grain_buffer: bool,
+    atomics: bool,
 }
 
 unsafe impl<'a, T: Send> Send for SvmRawVec<'a, T> {}
@@ -51,11 +56,13 @@ impl<'a, T> SvmRawVec<'a, T> {
         assert!(!fine_grain_system, "SVM supports system memory, use Vec!");
 
         let fine_grain_buffer: bool = svm_capabilities & CL_DEVICE_SVM_FINE_GRAIN_BUFFER != 0;
+        let atomics: bool = fine_grain_buffer && (svm_capabilities & CL_DEVICE_SVM_ATOMICS != 0);
         SvmRawVec {
             ptr: ptr::null_mut(),
             cap: 0,
             context,
             fine_grain_buffer,
+            atomics,
         }
     }
 
@@ -63,25 +70,25 @@ impl<'a, T> SvmRawVec<'a, T> {
         context: &'a Context,
         svm_capabilities: cl_device_svm_capabilities,
         capacity: usize,
-    ) -> Self {
+    ) -> Result<Self> {
         let mut v = Self::new(context, svm_capabilities);
-        v.grow(capacity);
+        v.grow(capacity)?;
 
-        v
+        Ok(v)
     }
 
     fn with_capacity_zeroed(
         context: &'a Context,
         svm_capabilities: cl_device_svm_capabilities,
         capacity: usize,
-    ) -> Self {
-        let mut v = Self::with_capacity(context, svm_capabilities, capacity);
+    ) -> Result<Self> {
+        let mut v = Self::with_capacity(context, svm_capabilities, capacity)?;
         v.zero(capacity);
 
-        v
+        Ok(v)
     }
 
-    fn grow(&mut self, count: usize) {
+    fn grow(&mut self, count: usize) -> Result<()> {
         let elem_size = mem::size_of::<T>();
 
         let mut new_cap = count;
@@ -96,7 +103,11 @@ impl<'a, T> SvmRawVec<'a, T> {
         assert!(size <= (isize::MAX as usize) / 2, "capacity overflow");
 
         let svm_mem_flags: cl_svm_mem_flags = if self.fine_grain_buffer {
-            CL_MEM_SVM_FINE_GRAIN_BUFFER | CL_MEM_READ_WRITE
+            if self.atomics {
+                CL_MEM_SVM_FINE_GRAIN_BUFFER | CL_MEM_READ_WRITE | CL_MEM_SVM_ATOMICS
+            } else {
+                CL_MEM_SVM_FINE_GRAIN_BUFFER | CL_MEM_READ_WRITE
+            }
         } else {
             CL_MEM_READ_WRITE
         };
@@ -106,9 +117,7 @@ impl<'a, T> SvmRawVec<'a, T> {
             svm_mem_flags,
             size,
             alignment as cl_uint,
-        )
-        .expect("Error: clSVMAlloc");
-        assert!(!ptr.is_null(), "svm_alloc failed");
+        )?;
 
         // reallocation, copy old data to new pointer and free old memory
         if 0 < self.cap {
@@ -118,6 +127,8 @@ impl<'a, T> SvmRawVec<'a, T> {
 
         self.ptr = ptr as *mut T;
         self.cap = new_cap;
+
+        Ok(())
     }
 
     fn zero(&mut self, count: usize) {
@@ -134,9 +145,73 @@ impl<'a, T> Drop for SvmRawVec<'a, T> {
     }
 }
 
-/// An OpenCL Shared Virtual Memory (SVM) vector.  
+/// An OpenCL Shared Virtual Memory (SVM) vector.
 /// It has the lifetime of the [Context] that it was constructed from.  
 /// Note: T cannot be a "zero sized type" (ZST).
+///
+/// There are three types of Shared Virtual Memory:
+/// - CL_DEVICE_SVM_COARSE_GRAIN_BUFFER: OpenCL buffer memory objects can be shared.
+/// - CL_DEVICE_SVM_FINE_GRAIN_BUFFER: individual memory objects in an OpenCL buffer can be shared.
+/// - CL_DEVICE_SVM_FINE_GRAIN_SYSTEM: individual memory objects *anywhere* in **host** memory can be shared.
+///
+/// This `SvmVec` struct is designed to support CL_DEVICE_SVM_COARSE_GRAIN_BUFFER
+/// and CL_DEVICE_SVM_FINE_GRAIN_BUFFER.  
+/// A [Context] that supports CL_DEVICE_SVM_FINE_GRAIN_SYSTEM can (and should!)
+/// use a standard Rust vector instead.
+///
+/// Intel provided an excellent overview of Shared Virtual Memory here:
+/// [OpenCL 2.0 Shared Virtual Memory Overview](https://software.intel.com/content/www/us/en/develop/articles/opencl-20-shared-virtual-memory-overview.html).  
+/// A PDF version is available here: [SVM Overview](../docs/svmoverview.pdf).
+///
+/// To summarise, a CL_DEVICE_SVM_COARSE_GRAIN_BUFFER requires the SVM to be *mapped*
+/// before being read or written by the host and *unmapped* afterward, while
+/// CL_DEVICE_SVM_FINE_GRAIN_BUFFER can be used like a standard Rust vector.
+///
+/// The `is_fine_grained method` can be used to determine whether an `SvmVec` supports
+/// CL_DEVICE_SVM_FINE_GRAIN_BUFFER and should be used to control SVM map and unmap
+/// operations, e.g.:
+/// ```no_run
+/// # use cl3::device::CL_DEVICE_TYPE_GPU;
+/// # use opencl3::command_queue::CommandQueue;
+/// # use opencl3::context::Context;
+/// # use opencl3::device::Device;
+/// # use opencl3::kernel::{ExecuteKernel, Kernel};
+/// # use opencl3::memory::{CL_MAP_WRITE};
+/// # use opencl3::platform::get_platforms;
+/// # use opencl3::svm::SvmVec;
+/// # use opencl3::types::*;
+/// # use opencl3::Result;
+///
+/// # fn main() -> Result<()> {
+/// # let platforms = get_platforms().unwrap();
+/// # let devices = platforms[0].get_devices(CL_DEVICE_TYPE_GPU).unwrap();
+/// # let device = Device::new(devices[0]);
+/// # let context = Context::from_device(&device).unwrap();
+/// # let queue = CommandQueue::create(&context, context.default_device(), 0).unwrap();
+/// // The input data
+/// const ARRAY_SIZE: usize = 8;
+/// let value_array: [cl_int; ARRAY_SIZE] = [3, 2, 5, 9, 7, 1, 4, 2];
+///
+/// // Create an OpenCL SVM vector
+/// let mut test_values = SvmVec::<cl_int>::allocate(&context, ARRAY_SIZE)?;
+///
+/// // Map test_values if not an CL_MEM_SVM_FINE_GRAIN_BUFFER
+/// if !test_values.is_fine_grained() {
+///     queue.enqueue_svm_map(CL_BLOCKING, CL_MAP_WRITE, &mut test_values, &[])?;
+/// }
+///
+/// // Copy input data into the OpenCL SVM vector
+/// test_values.clone_from_slice(&value_array);
+///
+/// // Unmap test_values if not an CL_MEM_SVM_FINE_GRAIN_BUFFER
+/// if !test_values.is_fine_grained() {
+///     let unmap_test_values_event = queue.enqueue_svm_unmap(&test_values, &[])?;
+///     unmap_test_values_event.wait()?;
+/// }
+/// # Ok(())
+/// # }
+/// ```
+
 pub struct SvmVec<'a, T> {
     buf: SvmRawVec<'a, T>,
     len: usize,
@@ -157,121 +232,206 @@ impl<'a, T> SvmVec<'a, T> {
         self.len
     }
 
-    pub unsafe fn set_len(&mut self, new_len: usize) {
+    /// Whether the vector is empty
+    pub fn is_empty(&self) -> bool {
+        self.len == 0
+    }
+
+    /// Whether the vector is fine grained
+    pub fn is_fine_grained(&self) -> bool {
+        self.buf.fine_grain_buffer
+    }
+
+    /// Whether the vector can use atomics
+    pub fn has_atomics(&self) -> bool {
+        self.buf.atomics
+    }
+
+    /// Clear the vector, i.e. empty it.
+    pub fn clear(&mut self) {
+        self.len = 0;
+    }
+
+    /// Set the length of the vector.
+    /// If new_len > len, the new memory will be uninitialised.
+    ///
+    /// # Safety
+    /// May fail to grow buf if memory is not available for new_len.
+    pub fn set_len(&mut self, new_len: usize) -> Result<()> {
         if self.cap() < new_len {
-            self.buf.grow(new_len);
+            self.buf.grow(new_len)?;
         }
         self.len = new_len;
+        Ok(())
     }
 
-    /// Construct an empty SvmVec from a [Context] and the svm_capabilities of
-    /// the device (or devices) in the [Context].  
+    /// Construct an empty SvmVec from a [Context].  
     /// The SvmVec has the lifetime of the [Context].
-    pub fn new(context: &'a Context, svm_capabilities: cl_device_svm_capabilities) -> Self {
+    ///
+    /// # Panics
+    ///
+    /// The cl_device_svm_capabilities of the [Context] must include
+    /// CL_DEVICE_SVM_COARSE_GRAIN_BUFFER or CL_DEVICE_SVM_FINE_GRAIN_BUFFER.  
+    /// The cl_device_svm_capabilities must *not* include CL_DEVICE_SVM_FINE_GRAIN_SYSTEM,
+    /// a standard Rust `Vec!` should be used instead.
+    pub fn new(context: &'a Context) -> Self {
+        let svm_capabilities = context.get_svm_mem_capability();
         SvmVec {
-            buf: SvmRawVec::new(&context, svm_capabilities),
+            buf: SvmRawVec::new(context, svm_capabilities),
             len: 0,
         }
     }
 
-    pub fn with_capacity(
-        context: &'a Context,
-        svm_capabilities: cl_device_svm_capabilities,
-        capacity: usize,
-    ) -> Self {
-        SvmVec {
-            buf: SvmRawVec::with_capacity(&context, svm_capabilities, capacity),
+    /// Construct an SvmVec with the given len of values from a [Context].
+    ///
+    /// returns a Result containing an SvmVec with len values of **uninitialised**
+    /// memory, or the OpenCL error.
+    ////
+    /// # Panics
+    ///
+    /// The cl_device_svm_capabilities of the [Context] must include
+    /// CL_DEVICE_SVM_COARSE_GRAIN_BUFFER or CL_DEVICE_SVM_FINE_GRAIN_BUFFER.  
+    /// The cl_device_svm_capabilities must *not* include CL_DEVICE_SVM_FINE_GRAIN_SYSTEM,
+    /// a standard Rust `Vec!` should be used instead.
+    pub fn allocate(context: &'a Context, len: usize) -> Result<Self> {
+        let svm_capabilities = context.get_svm_mem_capability();
+        Ok(SvmVec {
+            buf: SvmRawVec::with_capacity(context, svm_capabilities, len)?,
+            len,
+        })
+    }
+
+    /// Construct an empty SvmVec with the given capacity from a [Context].
+    ///
+    /// returns a Result containing an empty SvmVec, or the OpenCL error.
+    ///
+    /// # Panics
+    ///
+    /// The cl_device_svm_capabilities of the [Context] must include
+    /// CL_DEVICE_SVM_COARSE_GRAIN_BUFFER or CL_DEVICE_SVM_FINE_GRAIN_BUFFER.  
+    /// The cl_device_svm_capabilities must *not* include CL_DEVICE_SVM_FINE_GRAIN_SYSTEM,
+    /// a standard Rust `Vec!` should be used instead.
+    pub fn with_capacity(context: &'a Context, capacity: usize) -> Result<Self> {
+        let svm_capabilities = context.get_svm_mem_capability();
+        Ok(SvmVec {
+            buf: SvmRawVec::with_capacity(context, svm_capabilities, capacity)?,
             len: 0,
-        }
+        })
     }
 
-    pub fn with_capacity_zeroed(
-        context: &'a Context,
-        svm_capabilities: cl_device_svm_capabilities,
-        capacity: usize,
-    ) -> Self {
-        SvmVec {
-            buf: SvmRawVec::with_capacity_zeroed(&context, svm_capabilities, capacity),
-            len: 0,
-        }
+    /// Construct an SvmVec with the given len of values from a [Context] and
+    /// the svm_capabilities of the device (or devices) in the [Context].
+    ///
+    /// # Panics
+    ///
+    /// The function will panic if the cl_device_svm_capabilities of the [Context]
+    /// does **not** include CL_DEVICE_SVM_FINE_GRAIN_BUFFER.
+    ///
+    /// returns a Result containing an SvmVec with len values of zeroed
+    /// memory, or the OpenCL error.
+    pub fn allocate_zeroed(context: &'a Context, len: usize) -> Result<Self> {
+        let svm_capabilities = context.get_svm_mem_capability();
+        let fine_grain_buffer: bool = svm_capabilities & CL_DEVICE_SVM_FINE_GRAIN_BUFFER != 0;
+        assert!(
+            fine_grain_buffer,
+            "SVM is not fine grained, use `allocate` instead."
+        );
+        Ok(SvmVec {
+            buf: SvmRawVec::with_capacity_zeroed(context, svm_capabilities, len)?,
+            len,
+        })
     }
 
-    /// Reserve vector capacity.
-    pub fn reserve(&mut self, capacity: usize) {
-        self.buf.grow(capacity);
+    /// Reserve vector capacity.  
+    /// returns an empty Result or the OpenCL error.
+    pub fn reserve(&mut self, capacity: usize) -> Result<()> {
+        self.buf.grow(capacity)
     }
 
+    /// Push a value onto the vector.
+    ///
+    /// # Panics
+    ///
+    /// The function will panic if a coarse grain buffer attempts to grow the vector.
     pub fn push(&mut self, elem: T) {
         if self.len == self.cap() {
-            self.buf.grow(self.len + 1);
+            assert!(
+                self.is_fine_grained(),
+                "SVM is not fine grained, cannot grow the vector."
+            );
+            self.buf.grow(self.len + 1).unwrap();
         }
 
         unsafe {
-            ptr::write(self.ptr().offset(self.len as isize), elem);
+            ptr::write(self.ptr().add(self.len), elem);
         }
 
         // Can't fail, we'll OOM first.
         self.len += 1;
     }
 
+    /// Pop a value from the vector.
     pub fn pop(&mut self) -> Option<T> {
         if self.len == 0 {
             None
         } else {
             self.len -= 1;
-            unsafe { Some(ptr::read(self.ptr().offset(self.len as isize))) }
+            unsafe { Some(ptr::read(self.ptr().add(self.len))) }
         }
     }
 
+    /// Insert a value into the vector at index.
+    ///
+    /// # Panics
+    ///
+    /// The function will panic if the index is out of bounds or
+    /// if a coarse grain buffer attempts to grow the vector.
     pub fn insert(&mut self, index: usize, elem: T) {
         assert!(index <= self.len, "index out of bounds");
         if self.cap() == self.len {
-            self.buf.grow(self.len + 1);
+            assert!(
+                self.is_fine_grained(),
+                "SVM is not fine grained, cannot grow the vector."
+            );
+            self.buf.grow(self.len + 1).unwrap();
         }
 
         unsafe {
             if index < self.len {
                 ptr::copy(
-                    self.ptr().offset(index as isize),
-                    self.ptr().offset(index as isize + 1),
+                    self.ptr().add(index),
+                    self.ptr().add(index + 1),
                     self.len - index,
                 );
             }
-            ptr::write(self.ptr().offset(index as isize), elem);
+            ptr::write(self.ptr().add(index), elem);
             self.len += 1;
         }
     }
 
+    /// Remove a value from the vector at index.
+    ///
+    /// # Panics
+    ///
+    /// The function will panic if the index is out of bounds.
     pub fn remove(&mut self, index: usize) -> T {
         assert!(index < self.len, "index out of bounds");
         unsafe {
             self.len -= 1;
-            let result = ptr::read(self.ptr().offset(index as isize));
+            let result = ptr::read(self.ptr().add(index));
             ptr::copy(
-                self.ptr().offset(index as isize + 1),
-                self.ptr().offset(index as isize),
+                self.ptr().add(index + 1),
+                self.ptr().add(index),
                 self.len - index,
             );
             result
         }
     }
 
-    pub fn into_iter(self) -> IntoIter<'a, T> {
-        unsafe {
-            let iter = RawValIter::new(&self);
-            let buf = ptr::read(&self.buf);
-            mem::forget(self);
-
-            IntoIter {
-                iter: iter,
-                _buf: buf,
-            }
-        }
-    }
-
+    /// Drain the vector.
     pub fn drain(&mut self) -> Drain<T> {
         unsafe {
-            let iter = RawValIter::new(&self);
+            let iter = RawValIter::new(self);
 
             // this is a mem::forget safety thing. If Drain is forgotten, we just
             // leak the whole Vec's contents. Also we need to do this *eventually*
@@ -279,16 +439,31 @@ impl<'a, T> SvmVec<'a, T> {
             self.len = 0;
 
             Drain {
-                iter: iter,
+                iter,
                 vec: PhantomData,
             }
         }
     }
 }
 
+impl<'a, T> IntoIterator for SvmVec<'a, T> {
+    type Item = T;
+    type IntoIter = IntoIter<'a, Self::Item>;
+
+    fn into_iter(self) -> Self::IntoIter {
+        unsafe {
+            let iter = RawValIter::new(&self);
+            let buf = ptr::read(&self.buf);
+            mem::forget(self);
+
+            Self::IntoIter { iter, _buf: buf }
+        }
+    }
+}
+
 impl<'a, T> Drop for SvmVec<'a, T> {
     fn drop(&mut self) {
-        while let Some(_) = self.pop() {}
+        while self.pop().is_some() {}
         // allocation is handled by SvmRawVec
     }
 }
@@ -325,10 +500,10 @@ impl<T> RawValIter<T> {
             start: slice.as_ptr(),
             end: if mem::size_of::<T>() == 0 {
                 ((slice.as_ptr() as usize) + slice.len()) as *const _
-            } else if slice.len() == 0 {
+            } else if slice.is_empty() {
                 slice.as_ptr()
             } else {
-                slice.as_ptr().offset(slice.len() as isize)
+                slice.as_ptr().add(slice.len())
             },
         }
     }
